% pop_sevenSegmentica() - An EEGLAB plugin to perform ICA demo. Uses EEGLAB
%                         functions to demonstrate unmixing using ICA using
%                         seven segment display, which is adopted from
%                         Jeremy Chen's code.
%
% How to run the demo:
%
%   1. Choose which numbers to mix (top row, check boxes.)
%   2. Set kurtosis, data length, and white noise level.
%   3. Press 'Mix them' button. This generates pink noises (normalized
%      variance) and assign them to each ground truth numbers. Kurtosis
%      modulcation is not applied to white nose.
%   4. Press 'Play the movie' if you want to see how the mixed numbers plus
%      noise should look like (samples50 equally separated points).
%   5. Press 'Decompose the mixture' to run extended informax with all EEGLAB
%      default options. You may turn on realtime update for every 10/20/50
%      steps by changing the setting from the popup menu right to the
%      'Decompose the mixture' button.
%   6. When calculation is done, geographics correlations of the independent
%      components are displayed at the bottom row. Residual variance is
%      evaluated, which is defined as (X1-X2)/X2 where X1 and X2 are both 
%      1x7 normalized values for each of the seven segment display, anx X1
%      is the decomposed results, and X2 is the selected ground truth.
%   7. The other figure automatically pops up where original channel and
%      independent component probability density functions are plotted. On
%      the top left, you can specify which ground truth to overlay upon the 
%      obtained independent component activation (the first 1000 points are
%      shown). Also, the correlation coefficient across all the data points
%      as well as percent variance accounted for (PVAF), which is defined
%      as 100-100*var(Y1-Y2)/var(Y2) where Y1 is ground truth time series
%      and Y2 is independent component activation.
% 
% How to evaluate the results:
% 
%    Source PDF kurtosis -- larger means more kurtotic and easier to
%                           decompose for ICA.
%
%    Data length in frame -- longer means easier for ICA to decompose. Note
%                            that SCCN has been traditionally recommending
%                            the rule of thum of (ch^2)*30, but this
%                            recommended data length does not seem to work
%                            in this 7-channel demo. You probably need much
%                            larger than that here.
%
%   Background white noise variance -- smaller means easier for ICA to
%                                      decompose. This white noise is
%                                      independetly generated for each
%                                      channel.
%  
%    min r.v. -- minimum residual variance. Smaller means more similar to the
%                ground truth. Note that 'minimum' means that the given
%                decomposition is compared against all ground truth numbers,
%                and minimum r.v. is reported. This is due to the fact that
%                ICA results does not automatically identify itself with one
%                of the ground truths.
%
%    r -- correlation coefficient between the obtained IC activation and the
%         ground truth time series. Larger means more similar to the ground truth.
%
%    PVAF -- percent variance accounted for (PVAF) which is defined as
%            100-100*var(Y1-Y2)/var(Y2) where Y1 is ground truth time series
%            and Y2 is independent component activation. Larger means more
%            similar to the ground truth. Note that negative PVAF values are
%            truncated to be 0.
% 
%
%
% Reference:
%
% pinknoise.m is writte by Hristo Zhivomirov. https://www.mathworks.com/matlabcentral/fileexchange/42919-pink-red-blue-and-violet-noise-generation-with-matlab?focused=7825007&tab=function
%    H. Zhivomirov. A Method for Colored Noise Generation. Romanian Journal of Acoustics and Vibration, ISSN: 1584-7284, Vol. XV, No. 1, pp. 14-19, 2018. (http://rjav.sra.ro/index.php/rjav/article/view/40/29)
%    Hristo Zhivomirov (2019). Pink, Red, Blue and Violet Noise Generation with Matlab (https://www.mathworks.com/matlabcentral/fileexchange/42919-pink-red-blue-and-violet-noise-generation-with-matlab), MATLAB Central File Exchange. Retrieved June 14, 2019.
%
% MBHTM.m is written by E. Cheynet. https://www.mathworks.com/matlabcentral/fileexchange/52193-non-gaussian-process-generation
%   Gurley, K. R., Tognarelli, M. A., & Kareem, A. (1997). Analysis and simulation tools for wind engineering. Probabilistic Engineering Mechanics, 12(1), 9-31. 
%   Denoël, V. (2005). Application des méthodes d'analyse stochastique à l’étude des effets du vent sur les structures du génie civil. Unpublished doctoral thesis, University of Liège. (in French, p. 229)
%
% digital_clock_v2.m is written by Jeremy Chen. https://www.mathworks.com/matlabcentral/fileexchange/26488-simple-digital-clock?s_tid=FX_rc2_behav

% History:
% 06/14/2019 Makoto. Created.

% Copyright (C) 2019, Makoto Miyakoshi. SCCN, INC, UCSD.
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1.07  USA



function varargout = pop_sevenSegmentica(varargin)
% POP_SEVENSEGMENTICA MATLAB code for pop_sevenSegmentica.fig
%      POP_SEVENSEGMENTICA, by itself, creates a new POP_SEVENSEGMENTICA or raises the existing
%      singleton*.
%
%      H = POP_SEVENSEGMENTICA returns the handle to a new POP_SEVENSEGMENTICA or the handle to
%      the existing singleton*.
%
%      POP_SEVENSEGMENTICA('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in POP_SEVENSEGMENTICA.M with the given input arguments.
%
%      POP_SEVENSEGMENTICA('Property','Value',...) creates a new POP_SEVENSEGMENTICA or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before pop_sevenSegmentica_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to pop_sevenSegmentica_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help pop_sevenSegmentica

% Last Modified by GUIDE v2.5 13-Jun-2019 18:57:12

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @pop_sevenSegmentica_OpeningFcn, ...
                   'gui_OutputFcn',  @pop_sevenSegmentica_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before pop_sevenSegmentica is made visible.
function pop_sevenSegmentica_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to pop_sevenSegmentica (see VARARGIN)

draw7SegNumbers(handles.disp0, 0, 1);
draw7SegNumbers(handles.disp1, 1, 1);
draw7SegNumbers(handles.disp2, 2, 1);
draw7SegNumbers(handles.disp3, 3, 1);
draw7SegNumbers(handles.disp4, 4, 1);
draw7SegNumbers(handles.disp5, 5, 1);
draw7SegNumbers(handles.disp6, 6, 1);
draw7SegNumbers(handles.disp7, 7, 1);
draw7SegNumbers(handles.disp8, 8, 1);
draw7SegNumbers(handles.disp9, 9, 1);

% Choose default command line output for pop_sevenSegmentica
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes pop_sevenSegmentica wait for user response (see UIRESUME)
% uiwait(handles.ICADEMO_Fig1);


% --- Outputs from this function are returned to the command line.
function varargout = pop_sevenSegmentica_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in checkbox0.
function checkbox0_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox0 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox0


% --- Executes on button press in checkbox2.
function checkbox2_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox2


% --- Executes on button press in checkbox3.
function checkbox3_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox3


% --- Executes on button press in checkbox4.
function checkbox4_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox4 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox4


% --- Executes on button press in checkbox5.
function checkbox5_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox5


% --- Executes on button press in checkbox6.
function checkbox6_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox6


% --- Executes on button press in checkbox7.
function checkbox7_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox7 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox7


% --- Executes on button press in checkbox8.
function checkbox8_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox8 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox8


% --- Executes on button press in checkbox9.
function checkbox9_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox9 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox9


% --- Executes on button press in checkboxNoise.
function checkboxNoise_Callback(hObject, eventdata, handles)
% hObject    handle to checkboxNoise (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkboxNoise


% --- Executes on button press in checkbox1.
function checkbox1_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox1



function whiteNoiseLevelEdit_Callback(hObject, eventdata, handles)
% hObject    handle to whiteNoiseLevelEdit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of whiteNoiseLevelEdit as text
%        str2double(get(hObject,'String')) returns contents of whiteNoiseLevelEdit as a double


% --- Executes during object creation, after setting all properties.
function whiteNoiseLevelEdit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to whiteNoiseLevelEdit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function kurtosisEdit_Callback(hObject, eventdata, handles)
% hObject    handle to kurtosisEdit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of kurtosisEdit as text
%        str2double(get(hObject,'String')) returns contents of kurtosisEdit as a double


% --- Executes during object creation, after setting all properties.
function kurtosisEdit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to kurtosisEdit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function kEdit_Callback(hObject, eventdata, handles)
% hObject    handle to kEdit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of kEdit as text
%        str2double(get(hObject,'String')) returns contents of kEdit as a double


% --- Executes during object creation, after setting all properties.
function kEdit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to kEdit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



% --- Executes on selection change in icaUpdatePopupmenu.
function icaUpdatePopupmenu_Callback(hObject, eventdata, handles)
% hObject    handle to icaUpdatePopupmenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns icaUpdatePopupmenu contents as cell array
%        contents{get(hObject,'Value')} returns selected item from icaUpdatePopupmenu


% --- Executes during object creation, after setting all properties.
function icaUpdatePopupmenu_CreateFcn(hObject, eventdata, handles)
% hObject    handle to icaUpdatePopupmenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in MixThemPushbutton.
function MixThemPushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to MixThemPushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Check if at least 1 signal is selected.
if      get(handles.checkbox1, 'value') == 0 & ...
        get(handles.checkbox2, 'value') == 0 & ...
        get(handles.checkbox3, 'value') == 0 & ...
        get(handles.checkbox4, 'value') == 0 & ...
        get(handles.checkbox5, 'value') == 0 & ...
        get(handles.checkbox6, 'value') == 0 & ...
        get(handles.checkbox7, 'value') == 0 & ...
        get(handles.checkbox8, 'value') == 0 & ...
        get(handles.checkbox9, 'value') == 0 & ...
        get(handles.checkbox0, 'value') == 0
    
    error('Choose at least one ground truth input.')
end

% Check if White Noise is non-zero.
noiseLevel = get(handles.whiteNoiseLevelEdit, 'String');
if isempty(noiseLevel)
    error('Noise level must be > 0');
end

% Generate 7-ch with noise.
noiseLevel = str2num(noiseLevel);
dataLength = str2num(get(handles.kEdit, 'string')); % ch^2*30 from SCCN rule of thumb.
tmpData = randn(7, dataLength);
tmpData = bsxfun(@rdivide, tmpData, std(tmpData, 0, 2));
tmpData = tmpData*noiseLevel;

% Figure out 7seg to index.
numDisp0_7segIdx = [1:6];
numDisp1_7segIdx = [2 3];
numDisp2_7segIdx = [1 2 4 5 7];
numDisp3_7segIdx = [1 2 3 4 7];
numDisp4_7segIdx = [2 3 6 7];
numDisp5_7segIdx = [1 3 4 6 7];
numDisp6_7segIdx = [1 3 4 5 6 7];
numDisp7_7segIdx = [1:3];
numDisp8_7segIdx = [1:7];
numDisp9_7segIdx = [1:4 6 7];

% Add signals to the noise.
originalSignals = zeros(10, dataLength);

if get(handles.checkbox0, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp0_7segIdx,:) = tmpData(numDisp0_7segIdx,:) + repmat(currentSignal, [length(numDisp0_7segIdx),1]);
    originalSignals(10,:) = currentSignal;
end

if get(handles.checkbox1, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp1_7segIdx,:) = tmpData(numDisp1_7segIdx,:) + repmat(currentSignal, [length(numDisp1_7segIdx),1]);
    originalSignals(1,:) = currentSignal;
end

if get(handles.checkbox2, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp2_7segIdx,:) = tmpData(numDisp2_7segIdx,:) + repmat(currentSignal, [length(numDisp2_7segIdx),1]);
    originalSignals(2,:) = currentSignal;
end

if get(handles.checkbox3, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp3_7segIdx,:) = tmpData(numDisp3_7segIdx,:) + repmat(currentSignal, [length(numDisp3_7segIdx),1]);
    originalSignals(3,:) = currentSignal;
end

if get(handles.checkbox4, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp4_7segIdx,:) = tmpData(numDisp4_7segIdx,:) + repmat(currentSignal, [length(numDisp4_7segIdx),1]);
    originalSignals(4,:) = currentSignal;
end

if get(handles.checkbox5, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp5_7segIdx,:) = tmpData(numDisp5_7segIdx,:) + repmat(currentSignal, [length(numDisp5_7segIdx),1]);
    originalSignals(5,:) = currentSignal;
end

if get(handles.checkbox6, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp6_7segIdx,:) = tmpData(numDisp6_7segIdx,:) + repmat(currentSignal, [length(numDisp6_7segIdx),1]);
    originalSignals(6,:) = currentSignal;
end

if get(handles.checkbox7, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp7_7segIdx,:) = tmpData(numDisp7_7segIdx,:) + repmat(currentSignal, [length(numDisp7_7segIdx),1]);
    originalSignals(7,:) = currentSignal;
end

if get(handles.checkbox8, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp8_7segIdx,:) = tmpData(numDisp8_7segIdx,:) + repmat(currentSignal, [length(numDisp8_7segIdx),1]);
    originalSignals(8,:) = currentSignal;
end

if get(handles.checkbox9, 'value')
    currentSignal = pinknoise(dataLength,1)'; 
    currentSignal = MBHTM(currentSignal, 0, str2num(get(handles.kurtosisEdit, 'String')));
    tmpData(numDisp9_7segIdx,:) = tmpData(numDisp9_7segIdx,:) + repmat(currentSignal, [length(numDisp9_7segIdx),1]);
    originalSignals(9,:) = currentSignal;
end

mixedData = tmpData;

% Normalize the data into [0-1].
minDataValue  = min(tmpData(:));
tmpData       = tmpData-minDataValue ;
maxDataValue  = max(tmpData(:));
mixedDataNorm = tmpData/maxDataValue;

% Plot time series.
offsetDC = repmat([0 1 2 3 4 5 6]', [1 length(mixedData)]);
plot(handles.mixedLinePlot, [mixedDataNorm+offsetDC]');
axis(handles.mixedLinePlot, 'ij')
xlim(handles.mixedLinePlot, [0 length(mixedData)])
ylim(handles.mixedLinePlot, [0 7])
set( handles.mixedLinePlot, 'xtick', [])
set( handles.mixedLinePlot, 'ytick', [0.5 1.5 2.5 3.5 4.5 5.5 6.5], 'yticklabel', [1 2 3 4 5 6 7])

% Plot the results of the first frame.
vlineHandle = line(handles.mixedLinePlot, [1 1], [0 7], 'color', [0 0 0], 'linestyle', '--');
draw7SegIndependently(handles.mixed7segDisplay, mixedDataNorm(:,1))
text(handles.mixed7segDisplay, 2.25, -0.5,  '1', 'color', [1 1 1])
text(handles.mixed7segDisplay, 4.35, -3.0,  '2', 'color', [1 1 1])
text(handles.mixed7segDisplay, 4.35, -9.0,  '3', 'color', [1 1 1])
text(handles.mixed7segDisplay, 2.25, -11.5, '4', 'color', [1 1 1])
text(handles.mixed7segDisplay, 0.25, -9.0,  '5', 'color', [1 1 1])
text(handles.mixed7segDisplay, 0.25, -3.0,  '6', 'color', [1 1 1])
text(handles.mixed7segDisplay, 2.25, -6.0,  '7', 'color', [1 1 1])

% Store the data.
sevenSegmentica.mixedData       = mixedData;
sevenSegmentica.mixedDataNorm   = mixedDataNorm;
sevenSegmentica.originalSignals = originalSignals;
assignin('base', 'sevenSegmenticaData', sevenSegmentica);



% --- Executes on button press in playTheMoviePushbutton.
function playTheMoviePushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to playTheMoviePushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Obtain precalculated data.
try
    sevenSegmenticaData = evalin('base', 'sevenSegmenticaData');
catch
    error('Please press ''Mix them'' button first.')
end
mixedData       = sevenSegmenticaData.mixedData;
mixedDataNorm   = sevenSegmenticaData.mixedDataNorm;
originalSignals = sevenSegmenticaData.originalSignals;

% Plot time series.
offsetDC = repmat([0 1 2 3 4 5 6]', [1 length(mixedDataNorm)]);
plot(handles.mixedLinePlot, [mixedDataNorm+offsetDC]');
axis(handles.mixedLinePlot, 'ij')
xlim(handles.mixedLinePlot, [0 length(mixedDataNorm)])
ylim(handles.mixedLinePlot, [0 7])
set( handles.mixedLinePlot, 'xtick', [])
set( handles.mixedLinePlot, 'ytick', [0.5 1.5 2.5 3.5 4.5 5.5 6.5], 'yticklabel', [1 2 3 4 5 6 7])

% Plot the results of the first frame.
vlineHandle = line(handles.mixedLinePlot, [1 1], [0 7], 'color', [0 0 0], 'linestyle', '--');
draw7SegIndependently(handles.mixed7segDisplay, mixedDataNorm(:,1))
text(handles.mixed7segDisplay, 2.25, -0.5,  '1', 'color', [1 1 1])
text(handles.mixed7segDisplay, 4.35, -3.0,  '2', 'color', [1 1 1])
text(handles.mixed7segDisplay, 4.35, -9.0,  '3', 'color', [1 1 1])
text(handles.mixed7segDisplay, 2.25, -11.5, '4', 'color', [1 1 1])
text(handles.mixed7segDisplay, 0.25, -9.0,  '5', 'color', [1 1 1])
text(handles.mixed7segDisplay, 0.25, -3.0,  '6', 'color', [1 1 1])
text(handles.mixed7segDisplay, 2.25, -6.0,  '7', 'color', [1 1 1])

% Move the bar over time series.
for frameIdx = 1:floor(length(mixedDataNorm)/50):length(mixedDataNorm)
    
    % Update the vertical line position.
    set(vlineHandle, 'XData', [frameIdx frameIdx], 'YData', [0 7]);
    
    % Draw 7-seg display.
    draw7SegIndependently(handles.mixed7segDisplay, mixedDataNorm(:,frameIdx))
    text(handles.mixed7segDisplay, 2.25, -0.5,  '1', 'color', [1 1 1])
    text(handles.mixed7segDisplay, 4.35, -3.0,  '2', 'color', [1 1 1])
    text(handles.mixed7segDisplay, 4.35, -9.0,  '3', 'color', [1 1 1])
    text(handles.mixed7segDisplay, 2.25, -11.5, '4', 'color', [1 1 1])
    text(handles.mixed7segDisplay, 0.25, -9.0,  '5', 'color', [1 1 1])
    text(handles.mixed7segDisplay, 0.25, -3.0,  '6', 'color', [1 1 1])
    text(handles.mixed7segDisplay, 2.25, -6.0,  '7', 'color', [1 1 1])
    
    % Draw now.
    drawnow
end



% --- Executes on button press in decomposeThemPushbutton.
function decomposeThemPushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to decomposeThemPushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Obtain precalculated data.
try
    sevenSegmenticaData = evalin('base', 'sevenSegmenticaData');
catch
    error('Please press ''Mix them'' button first.')
end
mixedData       = sevenSegmenticaData.mixedData;
mixedDataNorm   = sevenSegmenticaData.mixedDataNorm;
originalSignals = sevenSegmenticaData.originalSignals;

        % Perform high-pass filter.
        % EEG = eeg_emptyset;
        % EEG.data  = icaEducationalDemoData;
        % EEG.nbchan = size(EEG.data,1);
        % EEG.trials = 1;
        % EEG.srate = 100;
        % EEG.pnts  = length(EEG.data);
        % EEG = pop_eegfiltnew(EEG, [],1,330,1,[],0);
        % icaEducationalDemoData = EEG.data;
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Close the other figure. %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close(findobj('tag', 'sevenSegmenticaFigure2'))

%%%%%%%%%%%%%%%%
%%% Run ICA. %%%
%%%%%%%%%%%%%%%%
[icaweights,icasphere] = runicaSevenSegmentica(mixedData, 'extended', 1, 'updateIntervalIdx', get(handles.icaUpdatePopupmenu, 'value'));

% Obtain icawinv.
icawinv = pinv(icaweights*icasphere);

% Make icawinv positive and normalized to 0<=x<=1.
icawinvPlot = zeros(size(icawinv));
for icIdx = 1:length(icawinv)
    if min(icawinv(:,icIdx))<0
        icawinvPlot(:,icIdx) = icawinv(:,icIdx)-min(icawinv(:,icIdx));
    else
        icawinvPlot(:,icIdx) = icawinv(:,icIdx);
    end
end
icawinvPlot = bsxfun(@rdivide, icawinvPlot, max(icawinvPlot,[],1));

% Store the updated data.
sevenSegmenticaData.icaweights  = icaweights;
sevenSegmenticaData.icasphere   = icasphere;
sevenSegmenticaData.icawinv     = icawinv;
sevenSegmenticaData.icawinvPlot = icawinvPlot;
assignin('base', 'sevenSegmenticaData', sevenSegmenticaData);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Plot the final results. %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sevenSegNumbers = zeros(7,10);
sevenSegNumbers(:,1)  = [0 1 1 0 0 0 0]';
sevenSegNumbers(:,2)  = [1 1 0 1 1 0 1]';
sevenSegNumbers(:,3)  = [1 1 1 1 0 0 1]';
sevenSegNumbers(:,4)  = [0 1 1 0 0 1 1]';
sevenSegNumbers(:,5)  = [1 0 1 1 0 1 1]';
sevenSegNumbers(:,6)  = [1 0 1 1 1 1 1]';
sevenSegNumbers(:,7)  = [1 1 1 0 0 0 0]';
sevenSegNumbers(:,8)  = [1 1 1 1 1 1 1]';
sevenSegNumbers(:,9)  = [1 1 1 1 0 1 1]';
sevenSegNumbers(:,10) = [1 1 1 1 1 1 0]';

draw7SegIndependently(handles.ic1, icawinvPlot(:,1));
minRV = min(sum(bsxfun(@minus, sevenSegNumbers, icawinvPlot(:,1)).^2)/sum(icawinvPlot(:,1).^2));
if minRV > 1, minRV = 1; end
set(handles.rv1Text, 'String', sprintf('min r.v. %.1f%%', minRV*100));

draw7SegIndependently(handles.ic2, icawinvPlot(:,2));
minRV = min(sum(bsxfun(@minus, sevenSegNumbers, icawinvPlot(:,2)).^2)/sum(icawinvPlot(:,2).^2));
if minRV > 1, minRV = 1; end
set(handles.rv2Text, 'String', sprintf('min r.v. %.1f%%', minRV*100));

draw7SegIndependently(handles.ic3, icawinvPlot(:,3));
minRV = min(sum(bsxfun(@minus, sevenSegNumbers, icawinvPlot(:,3)).^2)/sum(icawinvPlot(:,3).^2));
if minRV > 1, minRV = 1; end
set(handles.rv3Text, 'String', sprintf('min r.v. %.1f%%', minRV*100));

draw7SegIndependently(handles.ic4, icawinvPlot(:,4));
minRV = min(sum(bsxfun(@minus, sevenSegNumbers, icawinvPlot(:,4)).^2)/sum(icawinvPlot(:,4).^2));
if minRV > 1, minRV = 1; end
set(handles.rv4Text, 'String', sprintf('min r.v. %.1f%%', minRV*100));

draw7SegIndependently(handles.ic5, icawinvPlot(:,5));
minRV = min(sum(bsxfun(@minus, sevenSegNumbers, icawinvPlot(:,5)).^2)/sum(icawinvPlot(:,5).^2));
if minRV > 1, minRV = 1; end
set(handles.rv5Text, 'String', sprintf('min r.v. %.1f%%', minRV*100));

draw7SegIndependently(handles.ic6, icawinvPlot(:,6));
minRV = min(sum(bsxfun(@minus, sevenSegNumbers, icawinvPlot(:,6)).^2)/sum(icawinvPlot(:,6).^2));
if minRV > 1, minRV = 1; end
set(handles.rv6Text, 'String', sprintf('min r.v. %.1f%%', minRV*100));

draw7SegIndependently(handles.ic7, icawinvPlot(:,7));
minRV = min(sum(bsxfun(@minus, sevenSegNumbers, icawinvPlot(:,7)).^2)/sum(icawinvPlot(:,7).^2));
if minRV > 1, minRV = 1; end
set(handles.rv7Text, 'String', sprintf('min r.v. %.1f%%', minRV*100));



%%%%%%%%%%%%%%%%%
%%% Plot PDF. %%% 
%%%%%%%%%%%%%%%%%

% Load the figure template.
plot_sevenSegmentica;

% Normalize the data.
mixedDataZscore       = zscore(mixedData,0,2);
originalSignalIdx     = find(var(originalSignals,0,2)>0);
originalSignalsZscore = zscore(originalSignals(originalSignalIdx,:),0,2);

% Compute PDF.
numChan = 7;
chPdf = zeros(numChan,100);
chBin = zeros(numChan,100);
for chIdx = 1:numChan
    [chPdf(chIdx,:), chBin(chIdx,:)] = ksdensity(mixedDataZscore(chIdx,:));
end

numIc = length(originalSignalIdx);
icPdf = zeros(numIc,100);
icBin = zeros(numIc,100);
for icIdx = 1:numIc
    [icPdf(icIdx,:), icBin(icIdx,:)] = ksdensity(originalSignalsZscore(icIdx,:));
end

% Plot PDF.
chPdfPlotHandle = findobj('tag', 'channelPdfAxes');
cla(chPdfPlotHandle)
hold(chPdfPlotHandle, 'on');
chHandleList   = [];
chHandleLegend = {};
for chIdx = 1:numChan
    currentHandle = plot(chPdfPlotHandle, chBin(icIdx,:), chPdf(chIdx,:));
    chHandleList   = [chHandleList currentHandle];
    chHandleLegend{1,chIdx} = ['Seg' num2str(chIdx)];
end
hold(chPdfPlotHandle, 'off');
xlim(chPdfPlotHandle, [-5 5])
ylim(chPdfPlotHandle, [0 1])
legend(chPdfPlotHandle, chHandleList, chHandleLegend)

icPdfPlotHandle = findobj('tag', 'icPdfAxes');
cla(icPdfPlotHandle)
hold(icPdfPlotHandle, 'on');
icHandleList   = [];
icHandleLegend = {};
for icIdx = 1:numIc
    currentHandle = plot(icPdfPlotHandle, icBin(icIdx,:), icPdf(icIdx,:));
    icHandleList   = [icHandleList currentHandle];
    icHandleLegend{1,icIdx} = ['IC' num2str(icIdx)];
end
hold(icPdfPlotHandle, 'off');
xlim(icPdfPlotHandle, [-5 5])
ylim(icPdfPlotHandle, [0 1])
legend(icPdfPlotHandle, icHandleList, icHandleLegend)



%%%%%%%%%%%%%%%%%%%%
%%% Plot icaact. %%% 
%%%%%%%%%%%%%%%%%%%%

% Show 7-seg ICA results also in the other figure.
draw7SegIndependently(findobj('tag', 'resultSeg1Axes'), icawinvPlot(:,1));
draw7SegIndependently(findobj('tag', 'resultSeg2Axes'), icawinvPlot(:,2));
draw7SegIndependently(findobj('tag', 'resultSeg3Axes'), icawinvPlot(:,3));
draw7SegIndependently(findobj('tag', 'resultSeg4Axes'), icawinvPlot(:,4));
draw7SegIndependently(findobj('tag', 'resultSeg5Axes'), icawinvPlot(:,5));
draw7SegIndependently(findobj('tag', 'resultSeg6Axes'), icawinvPlot(:,6));
draw7SegIndependently(findobj('tag', 'resultSeg7Axes'), icawinvPlot(:,7));

% Obtain icaact.
icaact = icaweights*icasphere*mixedData;

% Plot icaact.
timeSeriesAxesHandle = findobj('tag', 'timeSeriesAxes');

diminishingConstant = 10;

offsetDC = repmat([0.5 1.5 2.5 3.5 4.5 5.5 6.5]', [1 length(mixedDataNorm)]);
cla(timeSeriesAxesHandle)
plotHandle = plot(timeSeriesAxesHandle, [icaact/diminishingConstant+offsetDC]');
axis(timeSeriesAxesHandle, 'ij')
xlim(timeSeriesAxesHandle, [0 1000])
ylim(timeSeriesAxesHandle, [0 7])
set( timeSeriesAxesHandle, 'xtick', [])
set( timeSeriesAxesHandle, 'ytick', [0.5 1.5 2.5 3.5 4.5 5.5 6.5], 'yticklabel', [1 2 3 4 5 6 7])

hold(timeSeriesAxesHandle, 'on')

% Overlay the first Signal.
currentlySelectedGroundTruth  = originalSignals(originalSignalIdx(1),:);
sample7SegHandle = findobj('tag', 'sample7SegAxes');
draw7SegNumbers(sample7SegHandle ,originalSignalIdx(1), 1);
currentlySelectedSignalRepmat = repmat(currentlySelectedGroundTruth, [7 1]);
firstSignalHandles = plot(timeSeriesAxesHandle, [currentlySelectedSignalRepmat/diminishingConstant+offsetDC]', 'k');
for lineObjIdx = 1:length(firstSignalHandles)
    set(firstSignalHandles(lineObjIdx), 'tag', ['overlaySignal' num2str(lineObjIdx)]);
end


% Compute correlation coefficient and PVAF.
ic1TextHandle = findobj('tag', 'ic1LineText');
corrCoeffMatrix = corr([icaact(1,:)' currentlySelectedGroundTruth']);
corrCoef = corrCoeffMatrix(1,2);
PVAF = 100-100*var(icaact(1,:)-currentlySelectedGroundTruth)/var(currentlySelectedGroundTruth);
if PVAF<0, PVAF=0; end
set(ic1TextHandle, 'string', sprintf('r=%.2f, PVAF=%.0f%%', corrCoef, PVAF));

ic2TextHandle = findobj('tag', 'ic2LineText');
corrCoeffMatrix = corr([icaact(2,:)' currentlySelectedGroundTruth']);
corrCoef = corrCoeffMatrix(1,2);
PVAF = 100-100*var(icaact(2,:)-currentlySelectedGroundTruth)/var(currentlySelectedGroundTruth);
if PVAF<0, PVAF=0; end
set(ic2TextHandle, 'string', sprintf('r=%.2f, PVAF=%.0f%%', corrCoef, PVAF));

ic3TextHandle = findobj('tag', 'ic3LineText');
corrCoeffMatrix = corr([icaact(3,:)' currentlySelectedGroundTruth']);
corrCoef = corrCoeffMatrix(1,2);
PVAF = 100-100*var(icaact(3,:)-currentlySelectedGroundTruth)/var(currentlySelectedGroundTruth);
if PVAF<0, PVAF=0; end
set(ic3TextHandle, 'string', sprintf('r=%.2f, PVAF=%.0f%%', corrCoef, PVAF));

ic4TextHandle = findobj('tag', 'ic4LineText');
corrCoeffMatrix = corr([icaact(4,:)' currentlySelectedGroundTruth']);
corrCoef = corrCoeffMatrix(1,2);
PVAF = 100-100*var(icaact(4,:)-currentlySelectedGroundTruth)/var(currentlySelectedGroundTruth);
if PVAF<0, PVAF=0; end
set(ic4TextHandle, 'string', sprintf('r=%.2f, PVAF=%.0f%%', corrCoef, PVAF));

ic5TextHandle = findobj('tag', 'ic5LineText');
corrCoeffMatrix = corr([icaact(5,:)' currentlySelectedGroundTruth']);
corrCoef = corrCoeffMatrix(1,2);
PVAF = 100-100*var(icaact(5,:)-currentlySelectedGroundTruth)/var(currentlySelectedGroundTruth);
if PVAF<0, PVAF=0; end
set(ic5TextHandle, 'string', sprintf('r=%.2f, PVAF=%.0f%%', corrCoef, PVAF));

ic6TextHandle = findobj('tag', 'ic6LineText');
corrCoeffMatrix = corr([icaact(6,:)' currentlySelectedGroundTruth']);
corrCoef = corrCoeffMatrix(1,2);
PVAF = 100-100*var(icaact(6,:)-currentlySelectedGroundTruth)/var(currentlySelectedGroundTruth);
if PVAF<0, PVAF=0; end
set(ic6TextHandle, 'string', sprintf('r=%.2f, PVAF=%.0f%%', corrCoef, PVAF));

ic7TextHandle = findobj('tag', 'ic7LineText');
corrCoeffMatrix = corr([icaact(7,:)' currentlySelectedGroundTruth']);
corrCoef = corrCoeffMatrix(1,2);
PVAF = 100-100*var(icaact(7,:)-currentlySelectedGroundTruth)/var(currentlySelectedGroundTruth);
if PVAF<0, PVAF=0; end
set(ic7TextHandle, 'string', sprintf('r=%.2f, PVAF=%.0f%%', corrCoef, PVAF));
